from collections.abc import Mapping, Sequence
from typing import Annotated, overload

from numpy.typing import ArrayLike


class AudioCodec:
    @property
    def name(self) -> str: ...

class AudioPackets:
    def __repr__(self) -> str: ...

    @property
    def timestamp(self) -> tuple[float, float] | None: ...

    @property
    def sample_rate(self) -> int: ...

    @property
    def num_channels(self) -> int: ...

    @property
    def codec(self) -> AudioCodec: ...

    def clone(self) -> AudioPackets: ...

class CPUBuffer:
    @property
    def __array_interface__(self) -> dict: ...

class CPUStorage:
    pass

class DecodeConfig:
    def __init__(self, decoder: str | None = None, decoder_options: Mapping[str, str] | None = None) -> None: ...

class DemuxConfig:
    def __init__(self, format: str | None = None, format_options: Mapping[str, str] | None = None, buffer_size: int = 8096) -> None: ...

class Demuxer:
    def demux_audio(self, window: tuple[float, float] | None = None, bsf: str | None = None) -> AudioPackets: ...

    def demux_video(self, window: tuple[float, float] | None = None, bsf: str | None = None) -> VideoPackets: ...

    def demux_image(self, bsf: str | None = None) -> ImagePackets: ...

    def has_audio(self) -> bool: ...

    @property
    def audio_codec(self) -> AudioCodec: ...

    @property
    def video_codec(self) -> VideoCodec: ...

    @property
    def image_codec(self) -> ImageCodec: ...

    def streaming_demux_video(self, num_packets: int, bsf: str | None = None) -> StreamingVideoDemuxer: ...

class EncodeConfig:
    def __init__(self, muxer: str | None = None, muxer_options: Mapping[str, str] | None = None, encoder: str | None = None, encoder_options: Mapping[str, str] | None = None, format: str | None = None, width: int = -1, height: int = -1, scale_algo: str | None = None, filter_desc: str | None = None, bit_rate: int = -1, compression_level: int = -1, qscale: int = -1, gop_size: int = -1, max_bframes: int = -1) -> None: ...

class FFmpegAudioFrames:
    @property
    def num_frames(self) -> int: ...

    @property
    def sample_rate(self) -> int: ...

    @property
    def num_channels(self) -> int: ...

    @property
    def sample_fmt(self) -> str: ...

    def __len__(self) -> int: ...

    def __repr__(self) -> str: ...

    def clone(self) -> FFmpegAudioFrames: ...

class FFmpegImageFrames:
    @property
    def num_planes(self) -> int: ...

    @property
    def width(self) -> int: ...

    @property
    def height(self) -> int: ...

    @property
    def pix_fmt(self) -> str: ...

    @property
    def metadata(self) -> dict[str, str]: ...

    def __repr__(self) -> str: ...

    def clone(self) -> FFmpegImageFrames: ...

    @property
    def pts(self) -> float: ...

class FFmpegVideoFrames:
    @property
    def num_frames(self) -> int: ...

    @property
    def num_planes(self) -> int: ...

    @property
    def width(self) -> int: ...

    @property
    def height(self) -> int: ...

    @property
    def pix_fmt(self) -> str: ...

    def __len__(self) -> int: ...

    @overload
    def __getitem__(self, arg: slice, /) -> FFmpegVideoFrames: ...

    @overload
    def __getitem__(self, arg: int, /) -> FFmpegImageFrames: ...

    @overload
    def __getitem__(self, arg: Sequence[int], /) -> FFmpegVideoFrames: ...

    def __repr__(self) -> str: ...

    def clone(self) -> FFmpegVideoFrames: ...

class ImageCodec:
    @property
    def name(self) -> str: ...

class ImagePackets:
    @property
    def pix_fmt(self) -> str: ...

    @property
    def width(self) -> int: ...

    @property
    def height(self) -> int: ...

    @property
    def codec(self) -> ImageCodec: ...

    def __repr__(self) -> str: ...

    def clone(self) -> ImagePackets: ...

class InternalError(AssertionError):
    pass

class StreamingVideoDecoder:
    def decode(self, arg: int, /) -> FFmpegVideoFrames | None: ...

class StreamingVideoDemuxer:
    def done(self) -> bool: ...

    def next(self) -> VideoPackets: ...

class TracingSession:
    def init(self) -> None: ...

    def config(self, arg: str, /) -> None: ...

    def start(self, arg0: int, arg1: int, /) -> None: ...

    def stop(self) -> None: ...

class VideoCodec:
    @property
    def name(self) -> str: ...

class VideoPackets:
    @property
    def timestamp(self) -> tuple[float, float] | None: ...

    @property
    def pix_fmt(self) -> str: ...

    @property
    def width(self) -> int: ...

    @property
    def height(self) -> int: ...

    @property
    def frame_rate(self) -> tuple[int, int]: ...

    @property
    def codec(self) -> VideoCodec: ...

    def __len__(self) -> int: ...

    def __repr__(self) -> str: ...

    def clone(self) -> VideoPackets: ...

def apply_bsf(packets: VideoPackets, bsf: str) -> VideoPackets: ...

def convert_array(vals: Annotated[ArrayLike, dict(dtype='int64', order='C', device='cpu')], storage: CPUStorage | None = None) -> CPUBuffer: ...

@overload
def convert_frames(frames: FFmpegAudioFrames, storage: CPUStorage | None = None) -> CPUBuffer: ...

@overload
def convert_frames(frames: FFmpegVideoFrames, storage: CPUStorage | None = None) -> CPUBuffer: ...

@overload
def convert_frames(frames: FFmpegImageFrames, storage: CPUStorage | None = None) -> CPUBuffer: ...

@overload
def convert_frames(frames: Sequence[FFmpegAudioFrames], storage: CPUStorage | None = None) -> CPUBuffer: ...

@overload
def convert_frames(frames: Sequence[FFmpegVideoFrames], storage: CPUStorage | None = None) -> CPUBuffer: ...

@overload
def convert_frames(frames: Sequence[FFmpegImageFrames], storage: CPUStorage | None = None) -> CPUBuffer: ...

def cpu_storage(size: int) -> CPUStorage: ...

@overload
def decode_packets(packets: AudioPackets, *, decode_config: DecodeConfig | None = None, filter_desc: str | None = None) -> FFmpegAudioFrames: ...

@overload
def decode_packets(packets: VideoPackets, *, decode_config: DecodeConfig | None = None, filter_desc: str | None = None) -> FFmpegVideoFrames: ...

@overload
def decode_packets(packets: ImagePackets, *, decode_config: DecodeConfig | None = None, filter_desc: str | None = None) -> FFmpegImageFrames: ...

def encode_image(path: str, data: Annotated[ArrayLike, dict(order='C', device='cpu')], *, pix_fmt: str = 'rgb24', encode_config: EncodeConfig | None = None) -> None: ...

def get_ffmpeg_filters() -> list[str]: ...

def get_ffmpeg_log_level() -> int: ...

def get_ffmpeg_versions() -> dict[str, tuple[int, int, int]]: ...

def init_glog(arg: str, /) -> None: ...

def init_tracing() -> TracingSession: ...

def register_avdevices() -> None: ...

def set_ffmpeg_log_level(arg: int, /) -> None: ...

@overload
def trace_counter(arg0: int, arg1: int, /) -> None: ...

@overload
def trace_counter(arg0: int, arg1: float, /) -> None: ...

def trace_event_begin(arg: str, /) -> None: ...

def trace_event_end() -> None: ...
